#include "message.h"
#include <QVector>

Message::Message()
{

}

Message::Message(MessageType type){
    m_type = type;
}

void Message::setMessageType(const MessageType &mtype){
    m_type = mtype;
}

MessageType Message::getMessageType() const {
    return m_type;
}

std::vector<QString> Message::getDataStrings() const{
    return dataStrings;
}
std::vector<Peer> Message::getPeerVector() const{
    return peerVector;
}

void Message::insertDataString(const QString &string){

    dataStrings.push_back(string);
}

void Message::insertPeerObj(const Peer &peer){

    peerVector.push_back(peer);
}

std::vector<Room> Message::getRoomVector() const{
    return roomVector;
}
void Message::insertRoomObj(const Room &room){
    roomVector.push_back(room);
}

void Message::setDataStrings(const std::vector<QString> dataStrings_)
{
    dataStrings = dataStrings_;
}

void Message::setPeerVector(const std::vector<Peer> peerVector_)
{
    peerVector = peerVector_;
}

void Message::setRoomVector(const std::vector<Room> roomVector_)
{
    roomVector = roomVector_;
}

QDataStream & operator <<( QDataStream & stream, const MessageType &type){

    return stream <<(int) type;
}
QDataStream & operator >>(QDataStream & stream, MessageType & type){

    int enumVal;
    stream >> enumVal;
    type = MessageType(enumVal);
    return stream;
}



QDataStream & operator <<(QDataStream & stream, const Message &message){

    stream << message.getMessageType(); // writing enum value

    std::vector<QString> dataStrings = message.getDataStrings();
    int dataStringsSize = dataStrings.size();
    stream << dataStringsSize; // writing dataString vector size first
    std::vector<Peer> peerObjects = message.getPeerVector();
    int peerVectorSize = peerObjects.size();
    stream << peerVectorSize; // writing peer vector size
    std::vector<Room> roomVector = message.getRoomVector();
    int roomVectorSize = roomVector.size();
    stream << roomVectorSize; // writing room vector size
    // writing the actual dataStrings
    for( QString obj : dataStrings ){
        
        stream << obj;
    }
    // writing the actual peer objects
    for(Peer peer :  peerObjects){
        stream<<peer;

    }
    // writing the actual room objects
    for(Room room: roomVector){
        stream << room;
    }
    return stream;
}

QDataStream &  operator >>(QDataStream & stream, Message &message){
    MessageType mtype;
    QString stringObj;
    Peer peerObj;
    Room roomObj;
    int dataStringsSize;
    int peerVectorSize;
    int roomVectorSize;

    stream >> mtype; // reading enum object
    stream >> dataStringsSize; // reading dataString vector size
    stream >> peerVectorSize; // reading Peer vector size
    stream >> roomVectorSize; // reading Room vector size

    message.setMessageType(mtype);

    for(int i=0;i<dataStringsSize;i++){
        stream>>stringObj; // reading actual dataString
        message.insertDataString(stringObj);
    }
    for(int i=0;i<peerVectorSize; i++){
        stream>>peerObj; // reading actual peer objects
        message.insertPeerObj(peerObj);
    }
    for(int i=0;i<roomVectorSize; i++){
        stream >> roomObj; // reading actual room objects
        message.insertRoomObj(roomObj);
    }
    return stream;
}

